<?php
namespace kwy\wechatapi;

class WechatRequest
{

    protected $data_type='XML';
    protected $ssl=false;

    private $appid;
    private $mchid;
    private $key;
    private $ssl_cert_path;
    private $ssl_key_path;
    private $timeout=5;

    /**
     * @return mixed
     */
    public function getKey()
    {
        return $this->key;
    }

    /**
     * @param mixed $key
     */
    public function setKey($key)
    {
        $this->key = $key;
    }

    /**
     * @return string
     */
    public function getAppid()
    {
        return $this->appid;
    }

    /**
     * @param string $appid
     */
    public function setAppid($appid)
    {
        $this->appid = $appid;
    }

    /**
     * @return string
     */
    public function getMchid()
    {
        return $this->mchid;
    }

    /**
     * @param int $timeout
     */
    public function setTimeout($timeout)
    {
        $this->timeout = $timeout;
    }

    /**
     * @param string $mchid
     */
    public function setMchid($mchid)
    {
        $this->mchid = $mchid;
    }

    /**
     * @return mixed
     */
    public function getSslCertPath()
    {
        return $this->ssl_cert_path;
    }

    /**
     * @param mixed $ssl_cert_path
     */
    public function setSslCertPath($ssl_cert_path)
    {
        $this->ssl_cert_path = $ssl_cert_path;
    }

    /**
     * @return mixed
     */
    public function getSslKeyPath()
    {
        return $this->ssl_key_path;
    }

    /**
     * @param mixed $ssl_key_path
     */
    public function setSslKeyPath($ssl_key_path)
    {
        $this->ssl_key_path = $ssl_key_path;
    }




    protected function getBizResult($url,$biz_params){
        $biz_params['nonce_str']=$this->getNonceStr();
        $biz_params['sign']=$this->getSign($biz_params,WechatConfig::$key);
        $data=$this->arrayToXml($biz_params);
        $ret=$this->post($url,$data);
        $arr_ret=$this->xmlToArray($ret);
        if(isset($arr_ret['return_code'])&&$arr_ret['return_code']!=='SUCCESS'){
            throw new \Exception($arr_ret['return_msg']);
        }
        return $arr_ret;
    }

    private function getSign($params,$key){
        ksort($params);
        $str='';
        foreach($params as $k=>$v){
            $str.=$k.'='.$v.'&';
        }
        $str.='key='.$key;
        return strtoupper(md5($str));
    }

    private function post($url,$data){
        $ch=curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);//url
        curl_setopt($ch, CURLOPT_POST, 1);//post
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);//返回结果curl
        curl_setopt($ch, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4 );//强制ipv4解析
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER,0);      //忽略校验证书
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $this->timeout);

        //设置证书
        //使用证书：cert 与 key 分别属于两个.pem文件
        //默认格式为PEM，可以注释
        if($this->ssl){
            curl_setopt($ch,CURLOPT_SSLCERTTYPE,'PEM');
            curl_setopt($ch,CURLOPT_SSLCERT, $this->ssl_cert_path);
            //默认格式为PEM，可以注释
            curl_setopt($ch,CURLOPT_SSLKEYTYPE,'PEM');
            curl_setopt($ch,CURLOPT_SSLKEY, $this->ssl_key_path);
        }

        $ret = curl_exec($ch);
        if ($ret===false){
            throw new \Exception(curl_error($ch),curl_errno($ch));
        }
        return $ret;
    }

    private function arrayToXml($array){
        $xml = "<xml>";
        foreach ($array as $key=>$val)
        {
            $xml.="<".$key."><![CDATA[".$val."]]></".$key.">";
        }
        $xml.="</xml>";
        return $xml;
    }

    private function arrayToXmlNoCData($array){
        $xml = "<xml>";
        foreach ($array as $key=>$val)
        {
            $xml.="<".$key.">".$val."</".$key.">";
        }
        $xml.="</xml>";
        return $xml;
    }

    private function xmlToArray($xml){
        return json_decode(json_encode(simplexml_load_string($xml, 'SimpleXMLElement', LIBXML_NOCDATA)), true);
    }

    protected function getNonceStr($len=32){
        $chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        $str ="";
        for ( $i = 0; $i < $len; $i++ )  {
            $str.= substr($chars, mt_rand(0, strlen($chars)-1), 1);
        }
        return $str;
    }
}