<?php
/**
 * Created by PhpStorm.
 * User: roc
 * Date: 2019/12/19
 * Time: 17:01
 */
namespace kwy\wechatapi\openwx;

use kwy\util\http\CurlHttp;

class WxApp
{
    /**
     * @var CurlHttp
     */
    private $httpClient;

    /**
     * 第三方平台令牌
     * @var string
     */
    private $access_token = "";

    /**
     * 设置公众号平台令牌
     * @author roc
     * @datetime:2019/12/19 17:02
     * @param string $access_token
     * @return $this
     */
    public function setAccessToken($access_token){
        $this->access_token = $access_token;
        return $this;
    }

    /**
     * 获取第三方平台令牌
     * @author roc
     * @datetime:2019/12/19 17:03
     * @return string
     */
    public function getAccessToken(){
        return $this->access_token;
    }

    /**
     * 获取设置的行业信息
     * 获取帐号设置的行业信息。可登录微信公众平台，在公众号后台中查看行业信息。
     * 为方便第三方开发者，提供通过接口调用的方式来获取帐号所设置的行业信息，具体如下:
     * @author roc
     * @datetime:2019/12/20 18:17
     * @return mixed
     * @throws \Exception
     */
    public function getIndustry(){
        $data = [

        ];
        $url = "https://api.weixin.qq.com/cgi-bin/template/get_industry?access_token=".$this->access_token;

        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 设置所属行业
     * 设置行业可在微信公众平台后台完成，每月可修改行业1次，帐号仅可使用所属行业中相关的模板，
     * 为方便第三方开发者，提供通过接口调用的方式来修改账号所属行业，具体如下：
     * @author roc
     * @datetime:2019/12/20 18:17
     * @param int $industry_id1 公众号模板消息所属行业编号
     * @param int $industry_id2 公众号模板消息所属行业编号
     * @return mixed
     * @throws \Exception
     */
    public function apiSetIndustry($industry_id1,$industry_id2=0){
        $data = [
            "industry_id1" => $industry_id1
        ];
        if(!empty($industry_id2)) $data["industry_id2"] = $industry_id2;
        $url = "https://api.weixin.qq.com/cgi-bin/template/api_set_industry?access_token=".$this->access_token;

        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 获得模板ID
     * 从行业模板库选择模板到帐号后台，获得模板ID的过程可在微信公众平台后台完成。
     * 为方便第三方开发者，提供通过接口调用的方式来获取模板ID，具体如下：
     * @author roc
     * @datetime:2019/12/19 17:07
     * @param string $template_id_short 模板库中模板的编号，有“TM**”和“OPENTMTM**”等形式
     * @return mixed
     * @throws \Exception
     */
    public function apiAddTemplate($template_id_short){
        $data = [
            'template_id_short' => $template_id_short,
        ];
        $url = "https://api.weixin.qq.com/cgi-bin/template/api_add_template?access_token=".$this->access_token;

        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 发送模板消息
     * @author roc
     * @datetime:2019/12/20 16:34
     * @param string $openid
     * @param string $template_id
     * @param array $data
     *     @example ["first"=>["value"=>"恭喜你购买成功","color":"#173177"],"remark"=>["value"=>"备注1","color":"#173177"],keyword1]
     * @param string $url
     * @param array $miniprogram ["appid"=>"小程序appid","pagepath":"小程序页面路径"]
     *     @example ["appid"=>"xiaochengxuappid12345","pagepath":"index?foo=bar"]
     * @return mixed
     * @throws \Exception
     */
    public function sendTemplate($openid,$template_id,$data,$url="",$miniprogram=[]){
        $data = [
            'touser' => $openid,
            'template_id' => $template_id,
            'data' => $data
        ];
        if(!empty($url)) $data["url"] = $url;
        if(!empty($miniprogram)) $data["miniprogram"] = $miniprogram;

        $url = "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=".$this->access_token;
        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 获取模板列表
     * 获取已添加至帐号下所有模板列表，可在微信公众平台后台中查看模板列表信息。
     * 为方便第三方开发者，提供通过接口调用的方式来获取帐号下所有模板信息，具体如下:
     * @author roc
     * @datetime:2019/12/20 19:14
     * @return mixed
     * @throws \Exception
     */
    public function getAllPrivateTemplate(){
        $data = [];
        $url = "https://api.weixin.qq.com/cgi-bin/template/get_all_private_template?access_token=".$this->access_token;
        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 生成带参数的关注二维码
     * @author roc
     * @datetime:2020/3/19 16:20
     * @param string $type 二维码类型：QR_SCENE（临时的整型参数值）；QR_STR_SCENE（临时的字符串参数值）；QR_LIMIT_SCENE（永久的整型参数值）；QR_LIMIT_STR_SCENE（永久的字符串参数值）
     * @param int|string $scene 参数值
     * @param int $expire_seconds 过期时间（临时二维码专用）
     * @return mixed
     * @throws \Exception
     * @example {
    "ticket":"gQH47zwAAAAAAAAAAS5odHRwOi8vd2VpeGluLnFxLmNvbS9xLzAyZ0EwZDFBSFBmSDAxSkhIcjF1YzgAAgRrXnReAwQAjScA",
    "expire_seconds":2592000,
    "url":"http://weixin.qq.com/q/02gA0d1AHPfH01JHHr1uc8"
    }
     */
    public function createQrCode($type,$scene,$expire_seconds=2592000){
        $url = "https://api.weixin.qq.com/cgi-bin/qrcode/create?access_token=".$this->access_token;
        switch ($type){
            case "QR_SCENE":    //临时的整型参数值
                $data = [
                    "expire_seconds" => $expire_seconds,    //过期时间
                    "action_name" => "QR_SCENE",    //临时的整型参数值
                    "action_info" => [
                        "scene" => [
                            "scene_id"=>$scene
                        ]
                    ]
                ];
                break;
            case "QR_STR_SCENE":    //临时的字符串参数值
                $data = [
                    "expire_seconds" => $expire_seconds,    //
                    "action_name" => "QR_STR_SCENE",    //临时的整型参数值
                    "action_info" => [
                        "scene" => [
                            "scene_str"=>$scene
                        ]
                    ]
                ];
                break;
            case "QR_LIMIT_SCENE":  //永久的整型参数值
                $data = [
                    "action_name" => "QR_LIMIT_SCENE",    //临时的整型参数值
                    "action_info" => [
                        "scene" => [
                            "scene_id"=>$scene
                        ]
                    ]
                ];
                break;
            case "QR_LIMIT_STR_SCENE":  //永久的字符串参数值
                $data = [
                    "action_name" => "QR_LIMIT_STR_SCENE",    //永久的字符串参数值
                    "action_info" => [
                        "scene" => [
                            "scene_str"=>$scene
                        ]
                    ]
                ];
                break;
            default:
                throw new \Exception("错误的二维码类型");
        }
        //{"expire_seconds": 604800, "action_name": "QR_SCENE", "action_info": {"scene": {"scene_id": 123}}}
        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 设置服务器域名
     * @author roc
     * @datetime:2020/3/19 18:41
     * @param string $action 操作类型：add：添加；delete：删除；set：覆盖；get：获取
     * @param array $requestDomain
     * @param array $wsRequestDomain
     * @param array $uploadDomain
     * @param array $downloadDomain
     * @return mixed
     * @throws \Exception
     */
    public function setModifyDomain($action,$requestDomain=[],$wsRequestDomain=[],$uploadDomain=[],$downloadDomain=[]){
        $url = "https://api.weixin.qq.com/wxa/modify_domain?access_token=".$this->access_token;
        $data = [
            "action" => $action,

        ];
        if(!empty($requestDomain)){
            $data["requestdomain"] = $requestDomain;
        }
        if(!empty($wsRequestDomain)){
            $data["wsrequestdomain"] = $wsRequestDomain;
        }
        if(!empty($uploadDomain)){
            $data["uploaddomain"] = $uploadDomain;
        }
        if(!empty($downloadDomain)){
            $data["downloaddomain"] = $downloadDomain;
        }

        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 获取用户基本信息
     * @author roc
     * @datetime:2020/3/20 10:29
     * @param $openid
     * @return mixed
     * @throws \Exception
     * {
    "subscribe": 1,
    "openid": "oKzPrwOphnOVqbs4VpkVFpDNNC6E",
    "nickname": "鹏",
    "sex": 1,
    "language": "zh_CN",
    "city": "杭州",
    "province": "浙江",
    "country": "中国",
    "headimgurl": "http:\/\/thirdwx.qlogo.cn\/mmopen\/4hHfWZmiaHhb99WictZfODByvg939t4jI7e2zaECFFiaHfsWyFvSLqibYZpj2KbZyyV2nCtXgv7Pm1ktWbc1olDLFiaicNv9OvHuia0\/132",
    "subscribe_time": 1574305845,
    "remark": "",
    "groupid": 0,
    "tagid_list": [],
    "subscribe_scene": "ADD_SCENE_OTHERS",
    "qr_scene": 0,
    "qr_scene_str": ""
    }
     */
    public function getCustomerInfo($openid){
        $url = "https://api.weixin.qq.com/cgi-bin/user/info?access_token=".$this->access_token."&openid=".$openid."&lang=zh_CN";

        /**
         * 返回数据
         * {
        "authorizer_info": {
        "nick_name": "沪宁时代服务平台",
        "head_img": "http:\/\/wx.qlogo.cn\/mmopen\/W1hYCJNWG180eknPOLxuEfKxNPgr2joewJMn8RmqSmQ8hZyNPg6awvBbs1hJ7otDeYFZNk9G1IiaWM8XYxkT9ebHuaoxmDtt3\/0",
        "service_type_info": {
        "id": 2
        },
        "verify_type_info": {
        "id": 0
        },
        "user_name": "gh_56c70dec9ed1",
        "alias": "",
        "qrcode_url": "http:\/\/mmbiz.qpic.cn\/mmbiz_jpg\/a9lvM3o5Sm4bJDzMFbSQPkttr4m23PnBnfutK1FExiaxiaCiaoEiaBZ3aqtKo002yJIPEzgzA0lxKJqFUbcvxkIfGw\/0",
        "business_info": {
        "open_pay": 0,
        "open_shake": 0,
        "open_scan": 0,
        "open_card": 1,
        "open_store": 0
        },
        "idc": 1,
        "principal_name": "杭州萧山宁围沪宁副食品商店",
        "signature": "关注沪宁时代，即时收取特价商品介绍、门店活动、优惠券、电子会员、线上商城等。"
        },
        "authorization_info": {
        "authorizer_appid": "wx7a03580e7437b28f",
        "authorizer_refresh_token": "refreshtoken@@@E9l-O9_UfStFJkyng58yvbsKOt2Mh4t3CMj576o0s7w",
        "func_info": [{
        "funcscope_category": {
        "id": 1
        }
        }, {
        "funcscope_category": {
        "id": 4
        }
        }, {
        "funcscope_category": {
        "id": 7
        }
        }, {
        "funcscope_category": {
        "id": 2
        }
        }, {
        "funcscope_category": {
        "id": 3
        }
        }, {
        "funcscope_category": {
        "id": 11
        }
        }, {
        "funcscope_category": {
        "id": 5
        }
        }, {
        "funcscope_category": {
        "id": 8
        }
        }, {
        "funcscope_category": {
        "id": 13
        }
        }, {
        "funcscope_category": {
        "id": 9
        }
        }]
        }
        }
         */
        $result = $this->get($url);
        return $result;
    }

    ##region 一次性订阅模板消息相关

    /**
     * 获取当前帐号所设置的类目信息
     * @author roc
     * @datetime:2020/3/23 11:36
     * @return mixed
     * @throws \Exception
     * @example
     * {
    "data": [{
    "id": 670,
    "name": "线下超市/便利店"
    }],
    "errmsg": "ok",
    "errcode": 0
    }
     */
    public function getNewTmplCategory(){
        $url = "https://api.weixin.qq.com/wxaapi/newtmpl/getcategory?access_token=".$this->access_token;
        $result = $this->get($url);
        return $result;
    }

    /**
     * 获取小程序订阅消息的模板库标题列表
     * @author roc
     * @datetime:2020/3/23 13:41
     * @param string $ids 类目id，多个用逗号隔开
     * @param int $start
     * @param int $limit
     * @return mixed
     * @throws \Exception
     * @example
     * {
    "count": 123,
    "data": [{
    "categoryId": "670",
    "tid": 335,
    "title": "积分到账提醒",
    "type": 2
    }, {
    "categoryId": "670",
    "tid": 339,
    "title": "预订到货通知",
    "type": 2
    }],
    "errmsg": "ok",
    "errcode": 0
    }
     */
    public function getNewPubTemplateTitles($ids,$start=0,$limit=20){
        $url = "https://api.weixin.qq.com/wxaapi/newtmpl/getpubtemplatetitles?access_token=".$this->access_token;

        $url .= "&ids=".$ids;
        $url .= "&start=".$start;
        $url .= "&limit=".$limit;
        $result = $this->get($url);
        return $result;
    }

    /**
     * 获取模板标题下的关键词库
     * @author roc
     * @datetime:2020/3/23 11:36
     * @param int $id 模板标题 id，可通过接口获取，也可登录小程序后台查看获取
     * @return mixed
     * @throws \Exception
     * @example
     * {
    "data": [{
    "kid": 1,
    "name": "快递公司",
    "example": "海硕高铁速递",
    "rule": "thing"
    }, {
    "kid": 2,
    "name": "快递单号",
    "example": "EP136542054362",
    "rule": "character_string"
    }, {
    "kid": 3,
    "name": "发货时间",
    "example": "2019-11-1 09:29:26",
    "rule": "time"
    }, {
    "kid": 4,
    "name": "店铺名称",
    "example": "二胖鲜果旗舰店（天津和平津塔店）",
    "rule": "thing"
    }, {
    "kid": 5,
    "name": "订单商品",
    "example": "泰国小菠萝*1 金枕榴莲*2",
    "rule": "thing"
    }, {
    "kid": 6,
    "name": "取货码",
    "example": "123456",
    "rule": "number"
    }, {
    "kid": 7,
    "name": "订单编号",
    "example": "xjx20191212",
    "rule": "character_string"
    }, {
    "kid": 8,
    "name": "备注",
    "example": "天气原因，时间可能延长",
    "rule": "thing"
    }, {
    "kid": 9,
    "name": "收货地址",
    "example": "北京XX小区",
    "rule": "thing"
    }, {
    "kid": 10,
    "name": "商品数量",
    "example": "10",
    "rule": "number"
    }, {
    "kid": 11,
    "name": "订单金额",
    "example": "¥200元",
    "rule": "amount"
    }, {
    "kid": 12,
    "name": "发货状态",
    "example": "发货中",
    "rule": "phrase"
    }, {
    "kid": 13,
    "name": "下单时间",
    "example": "2019-09-12 13:00:00",
    "rule": "date"
    }, {
    "kid": 14,
    "name": "客户姓名",
    "example": "王二",
    "rule": "name"
    }, {
    "kid": 15,
    "name": "收货人电话",
    "example": "13000000000",
    "rule": "phone_number"
    }, {
    "kid": 16,
    "name": "配送方式",
    "example": "团长配送",
    "rule": "thing"
    }, {
    "kid": 17,
    "name": "收货方式",
    "example": "自提",
    "rule": "phrase"
    }],
    "errmsg": "ok",
    "errcode": 0
    }
     */
    public function getNewPubTemplateKeywords($id){
        $url = "https://api.weixin.qq.com/wxaapi/newtmpl/getpubtemplatekeywords?access_token=".$this->access_token;
        $url .= "&tid=".$id;
        $result = $this->get($url);
        return $result;
    }

    /**
     * 组合模板并添加到个人模板库
     * @author roc
     * @datetime:2020/3/23 11:36
     * @param int $tid 模板标题 id，可通过接口获取，也可登录小程序后台查看获取
     * @param array $kidList 模板标题 id，可通过接口获取，也可登录小程序后台查看获取
     * @param string $sceneDesc 服务场景描述，15个字以内
     * @return mixed
     * @throws \Exception
     * @example
     * array (
    'priTmplId' => 'C12-MKwe2Iysc8dyXEJC7g9De7Z3qy-8qgEulViVyiQ',
    'errmsg' => 'ok',
    'errcode' => 0,
    )
     */
    public function addNewTemplate($tid,$kidList,$sceneDesc){
        $url = "https://api.weixin.qq.com/wxaapi/newtmpl/addtemplate?access_token=".$this->access_token;
        $data = [
            "tid" => $tid,  //模板标题 id，可通过接口获取，也可登录小程序后台查看获取
            "kidList" => $kidList,  //开发者自行组合好的模板关键词列表，关键词顺序可以自由搭配（例如 [3,5,4] 或 [4,5,3]），最多支持5个，最少2个关键词组合
            "sceneDesc" => $sceneDesc   //服务场景描述，15个字以内
        ];
        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 获取帐号下的模板列表
     * @author roc
     * @datetime:2020/3/23 11:36
     * @return mixed
     * @throws \Exception
     * @example
     * {
    "data": [{
    "priTmplId": "C12-MKwe2Iysc8dyXEJC7g9De7Z3qy-8qgEulViVyiQ",
    "title": "订单配送通知",
    "content": "商品信息:{{thing8.DATA}}\n配送人:{{name4.DATA}}\n配送员电话:{{phone_number10.DATA}}\n温馨提示:{{thing11.DATA}}\n",
    "example": "商品信息:苹果无线耳机\n配送人:梁某某\n配送员电话:+86-0766-66888866\n温馨提示:点击进入查看详细物流信息\n",
    "type": 2
    }],
    "errmsg": "ok",
    "errcode": 0
    }
     */
    public function getNewTemplateList(){
        $url = "https://api.weixin.qq.com/wxaapi/newtmpl/gettemplate?access_token=".$this->access_token;
        $result = $this->get($url);
        return $result;
    }

    /**
     * 删除帐号下的某个模板
     * @author roc
     * @datetime:2020/3/23 14:49
     * @param string $priTmplId 要删除的模板id
     * @return mixed
     * @throws \Exception
     * @example
     * array (
    'errmsg' => 'ok',
    'errcode' => 0,
    )
     */
    public function deleteNewTemplateList($priTmplId){
        $url = "https://api.weixin.qq.com/wxaapi/newtmpl/deltemplate?access_token=".$this->access_token;
        $data = ["priTmplId"=>$priTmplId];
        $result = $this->post($url,$data);
        return $result;
    }
    ##endregion 一次性订阅模板消息相关

    ## 代码管理
    /**
     * 上传小程序代码
     * @author roc
     * @datetime:2020/2/17 21:56
     * @param string $template_id 代码库中的代码模版ID
     * @param array $ext 第三方自定义的配置
     * @param string $user_version 代码版本号
     * @param string $user_desc 代码描述
     * @return mixed
     * @throws \Exception
     * @example
     * array (
    'errcode' => 0,
    'errmsg' => 'ok',
    )
     */
    public function commitWxa($template_id,$ext,$user_version,$user_desc){
        $url = "https://api.weixin.qq.com/wxa/commit?access_token=".$this->access_token;
        $data = [
            "template_id" => $template_id,  //代码库中的代码模版ID
            "ext_json" => json_encode($ext),  //第三方自定义的配置
            "user_version" => $user_version,  //代码版本号，开发者可自定义（长度不要超过 64 个字符）
            "user_desc" => $user_desc,  //代码描述，开发者可自定义
        ];
        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 获取已上传的代码的页面列表
     * @author roc
     * @datetime:2020/2/17 22:02
     * @return mixed
     * {
    "errcode": 0,
    "errmsg": "ok",
    "category_list": [{
    "first_class": "商家自营",
    "second_class": "家电\/数码\/手机",
    "first_id": 304,
    "second_id": 311
    }]
    }
     * @throws \Exception
     */
    public function getMiniAppCategory(){
//        $url = "https://api.weixin.qq.com/wxaapi/newtmpl/getcategory?access_token=".$this->access_token;
        $url = "https://api.weixin.qq.com/wxa/get_category?access_token=".$this->access_token;
        $result = $this->get($url);
        return $result["category_list"];
    }

    /**
     * 获取已上传的代码的页面列表
     * @author roc
     * @datetime:2020/2/17 22:02
     * @return mixed
     * array (
    'errcode' => 0,
    'errmsg' => 'ok',
    'page_list' =>
    array (
    0 => 'pages/index/index',
    1 => 'pages/cart/cart',
    2 => 'pages/user/user',
    3 => 'pages/category/category',
    4 => 'pages/address/address',
    5 => 'pages/address/addressManage',
    6 => 'pages/coupon/coupon',
    7 => 'pages/coupon/couponcenter',
    8 => 'pages/topup/topup',
    9 => 'pages/membercode/membercode',
    10 => 'pages/order/order',
    11 => 'pages/order/orderDetails',
    12 => 'pages/order/code',
    13 => 'pages/cart/paysucceed',
    14 => 'pages/cart/placeorder',
    15 => 'pages/activity/activity',
    16 => 'pages/activity/special',
    17 => 'pages/cart/detail',
    18 => 'pages/login/login',
    19 => 'pages/search/search',
    ),
    )
     * @throws \Exception
     */
    public function getWxaPage(){
        $url = "https://api.weixin.qq.com/wxa/get_page?access_token=".$this->access_token;
        $result = $this->get($url);
        return $result;
    }

    /**
     * 获取的体验版二维码
     * @author roc
     * @datetime:2020/2/17 22:02
     * @param string $path 指定二维码扫码后直接进入指定页面并可同时带上参数
     * @return mixed
     * array (
    'errcode' => 0,
    'errmsg' => 'ok',
    'page_list' =>
    array (
    0 => 'pages/index/index',
    1 => 'pages/cart/cart',
    2 => 'pages/user/user',
    3 => 'pages/category/category',
    4 => 'pages/address/address',
    5 => 'pages/address/addressManage',
    6 => 'pages/coupon/coupon',
    7 => 'pages/coupon/couponcenter',
    8 => 'pages/topup/topup',
    9 => 'pages/membercode/membercode',
    10 => 'pages/order/order',
    11 => 'pages/order/orderDetails',
    12 => 'pages/order/code',
    13 => 'pages/cart/paysucceed',
    14 => 'pages/cart/placeorder',
    15 => 'pages/activity/activity',
    16 => 'pages/activity/special',
    17 => 'pages/cart/detail',
    18 => 'pages/login/login',
    19 => 'pages/search/search',
    ),
    )
     * @throws WxException
     */
    public function getQrcode($path=""){
        $url = "https://api.weixin.qq.com/wxa/get_qrcode?access_token=".$this->access_token;
        if(!empty($path)){
            $url .= "&path=".urlencode($path);
        }
        $result = $this->get($url,false);
        return $result;
    }

    /**
     * 提交审核
     * @author roc
     * @datetime:2020/3/16 11:18
     * @param array $item_list 审核项列表（选填，至多填写 5 项）
     * @param array $preview_info 预览信息（小程序页面截图和操作录屏）
     * @param string $version_desc 小程序版本说明和功能解释
     * @param string $feedback_info 反馈内容，至多 200 字
     * @param string $feedback_stuff 用 | 分割的 media_id 列表，至多 5 张图片, 可以通过新增临时素材接口上传而得到
     * @return mixed
     * @throws \Exception
     * {
    "errcode": 0,
    "errmsg": "ok",
    "auditid": 426796295
    }
     */
    public function submitAudit($item_list=[],$preview_info=[],$version_desc="",$feedback_info="",$feedback_stuff=""){
        $url = "https://api.weixin.qq.com/wxa/submit_audit?access_token=".$this->access_token;
        $data = [];

        if(!empty($item_list)){
            $data["item_list"] = $item_list;
        }
        if(!empty($preview_info)){
            $data["preview_info"] = $preview_info;
        }
        if(!empty($version_desc)){
            $data["version_desc"] = $version_desc;
        }
        if(!empty($feedback_info)){
            $data["feedback_info"] = $feedback_info;
        }
        if(!empty($feedback_stuff)){
            $data["feedback_stuff"] = $feedback_stuff;
        }
        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * 查询指定发布审核单的审核状态
     * @author roc
     * @datetime:2020/3/16 11:27
     * @param string $auditId 提交审核时获得的审核 id
     * @return mixed
     * @throws \Exception
     * {
    "errcode": 0,
    "errmsg": "ok",
    "status": 1,
    "reason": "帐号信息不合规范",   //当 status = 1 时，返回的拒绝原因; status = 4 时，返回的延后原因
    "screenshot": "xxx|yyy|zzz" //当审核被拒绝时，会返回审核失败的小程序截图示例。用 | 分隔的 media_id 的列表，可通过获取永久素材接口拉取截图内容
    }
     */
    public function getAuditStatus($auditId){
        $url = "https://api.weixin.qq.com/wxa/get_auditstatus?access_token=".$this->access_token;
        $data = ["auditid"=>$auditId];
        $result = $this->post($url,$data);
        return $result;
        //status状态：
        //          0	审核成功
        //          1	审核被拒绝
        //          2	审核中
        //          3	已撤回
        //          4	审核延后
    }

    /**
     * 查询最新一次提交的审核状态
     * @author roc
     * @datetime:2020/3/16 11:27
     * @return mixed
     * @throws \Exception
     * {
    "errcode": 0,
    "errmsg": "ok",
    "auditid": "1234567",
    "status": 1,    //审核状态
    "reason": "帐号信息不合规范",   //当审核被拒绝时，返回的拒绝原因
    "ScreenShot": "xx|yy|zz",   //当审核被拒绝时，会返回审核失败的小程序截图示例。用 | 分隔的 media_id 的列表，可通过获取永久素材接口拉取截图内容
    }

     */
    public function getLatestAuditStatus(){
        $url = "https://api.weixin.qq.com/wxa/get_latest_auditstatus?access_token=".$this->access_token;
        $result = $this->get($url);
        return $result;
        //status状态：
        //          0	审核成功
        //          1	审核被拒绝
        //          2	审核中
        //          3	已撤回
    }

    /**
     * 小程序审核撤回
     * @author roc
     * @datetime:2020/3/16 11:27
     * @return mixed
     * @throws \Exception
     * {
    "errcode": 0,
    "errmsg": "ok"
    }
     */
    public function undoCodeAudit(){
        $url = "https://api.weixin.qq.com/wxa/undocodeaudit?access_token=".$this->access_token;
        $result = $this->get($url);
        return $result;
    }

    /**
     * 发布已通过审核的小程序
     * @author roc
     * @datetime:2020/3/16 11:27
     * @return mixed
     * @throws \Exception
     * {
    "errcode": 0,
    "errmsg": "ok"
    }
     */
    public function release(){
        $url = "https://api.weixin.qq.com/wxa/release?access_token=".$this->access_token;
        $data = [];
        $result = $this->post($url,$data);
        return $result;
    }

    /**
     * post请求微信并返回array数据
     * @author roc
     * @datetime:2019/12/24 11:36
     * @param string $url 微信接口地址
     * @param array $data post请求的数据
     * @return mixed
     * @throws \Exception
     */
    private function post($url,$data){
        $wx_response = $this->getHttpClient()
            ->setUrl($url)
            ->post(json_encode($data));
        if(empty($wx_response)) throw new \Exception("微信返回的数据为空");

        $result = json_decode($wx_response,true);
        if(empty($result)){
            throw new \Exception("转数组失败");
        }
        if(!empty($result["errcode"])){
            throw new WxException($result);
        }
        return $result;
    }

    /**
     * get请求微信并返回array数据
     * @author roc
     * @datetime:2019/12/24 11:36
     * @param string $url 微信接口地址
     * @param bool $isReturnBaseData 是否返回原始数据
     * @return mixed
     * @throws \Exception
     */
    private function get($url,$isReturnDecodeData=true){
        $wx_response = $this->getHttpClient()
            ->setUrl($url)
            ->get($url);
        if($isReturnDecodeData === false) return $wx_response;
        if(empty($wx_response)) throw new \Exception("微信返回的数据为空");

        $result = json_decode($wx_response,true);
        if(empty($result)){
            throw new \Exception("转数组失败");
        }
        if(!empty($result["errcode"])){
            throw new WxException($result);
        }
        return $result;
    }

    /**
     * 设置代理
     * @author roc
     * @datetime:2019/12/8 14:37
     * @param $proxy
     * @return $this
     */
    public function setProxy($proxy){
        $this->getHttpClient()->setProxy($proxy);
        return $this;
    }

    /**
     * 获取http请求客户端
     * @author roc
     * @return CurlHttp
     */
    public function getHttpClient(){
        if (empty($this->httpClient)){
            $this->httpClient = new CurlHttp();
            //$this->httpClient->setProxy('127.0.0.1:8888');
            $this->httpClient->setContentType('application/json');
        }
        return $this->httpClient;
    }
}