<?php
/**
 *
 * Auther: heyu
 * DateTime: 2020-01-03 15:01
 *
 */

namespace kwy\wechatapi\pay;


use kwy\util\http\CurlHttp;

class WechatPay extends ApiRequest
{

    private $mch_id='';

    private $appid='';

    private $sub_mch_id='';

    private $sub_appid='';

    private $key='';

    private $ssl_cert_path='';

    private $ssl_key_path='';

    /**
     * @param string $ssl_cert_path
     */
    public function setSslCertPath($ssl_cert_path)
    {
        $this->ssl_cert_path = $ssl_cert_path;
    }

    /**
     * @param string $ssl_key_path
     */
    public function setSslKeyPath($ssl_key_path)
    {
        $this->ssl_key_path = $ssl_key_path;
    }

    /**
     * @param string $appid
     */
    public function setAppid($appid)
    {
        $this->appid = $appid;
    }

    /**
     * @param string $mch_id
     */
    public function setMchId($mch_id)
    {
        $this->mch_id = $mch_id;
    }

    /**
     * @param string $key
     */
    public function setKey($key)
    {
        $this->key = $key;
    }

    /**
     * @param string $sub_mch_id
     */
    public function setSubMchId(string $sub_mch_id)
    {
        $this->sub_mch_id = $sub_mch_id;
    }

    /**
     * @param string $sub_appid
     */
    public function setSubAppid($sub_appid)
    {
        $this->sub_appid = $sub_appid;
    }


    /**
     * 获取刷脸付设备授权信息
     * @author heyu 2020-01-19 11:13
     * @param string $store_id 门店编号， 由商户定义， 各门店唯一
     * @param string $store_name 门店名称，由商户定义。（可用于展示）
     * @param string $device_id 终端设备编号，由商户定义。
     * @param string $rawdata 设备初始化信息
     * @return array
     * @example [
     *  'authinfo'=>'sdasdasd',
     * 'expires_in'>7200
     * ]
     */
    public function facePayAuthInfo($store_id,$store_name,$device_id,$rawdata){
        $url='https://payapp.weixin.qq.com/face/get_wxpayface_authinfo';
        $params=[
            'store_id'=>$store_id,
            'store_name'=>$store_name,
            'device_id'=>$device_id,
            'rawdata'=>$rawdata,
            'appid'=>$this->appid,
            'mch_id'=>$this->mch_id,
            'now'=>time(),
            'version'=>'1',
            'nonce_str'=>$this->getNonceStr(16),
            'sign_type'=>'MD5',
        ];
        if (!empty($this->sub_mch_id)){
            $params['sub_mch_id']=$this->sub_mch_id;
        }

        $params['sign']=$this->getSign($params,$this->key);

        $this->getHttp()->setUrl($url);
        $xml=$this->getHttp()->post($this->toXml($params));
        $result = json_decode(json_encode(simplexml_load_string($xml, 'SimpleXMLElement', LIBXML_NOCDATA)), true);

        if ($result['return_code']!='SUCCESS'){
            throw new \Exception($result['return_msg']);
        }

        return $result;

    }

    /**
     * 校验异步通知消息签名
     * @author heyu 2020-02-05 16:56
     * @param string $xml
     * @return bool
     */
    public function verifyNotifyData($xml){
        return $this->verifySign($this->xmlToArray($xml),$this->key);
    }

    /**
     * 调起刷码支付
     * @author heyu 2020-01-19 11:07
     * @param string $no
     * @param int $total_fee
     * @param string $auth_code
     * @param string $body
     * @return array
     * @throws \Exception
     */
    public function microPay($no,$total_fee,$auth_code,$body){
        $url='https://api.mch.weixin.qq.com/pay/micropay';
        $params=[
            'appid'=>$this->appid,
            'mch_id'=>$this->mch_id,
            'nonce_str'=>$this->getNonceStr(16),
            'sign_type'=>'MD5',
            'body'=>$body,
            'out_trade_no'=>$no,
            'total_fee'=>$total_fee,
            'spbill_create_ip'=>'115.29.165.48',
            'auth_code'=>$auth_code,
        ];
        if (!empty($this->sub_mch_id)){
            $params['sub_mch_id']=$this->sub_mch_id;
        }

        $params['sign']=$this->getSign($params,$this->key);

        $this->getHttp()->setUrl($url);
        $xml=$this->getHttp()->post($this->toXml($params));
        return json_decode(json_encode(simplexml_load_string($xml, 'SimpleXMLElement', LIBXML_NOCDATA)), true);
    }

    /**
     * @author heyu 2020-02-04 16:52
     * @param $no
     * @param $total_fee
     * @param $openid
     * @param $body
     * @param $notify_url
     * @param string $sub_openid
     */
    public function jsPay($no,$total_fee,$openid,$body,$notify_url,$sub_openid=""){
        $url='https://api.mch.weixin.qq.com/pay/unifiedorder';
        $params=[
            'appid'=>$this->appid,
            'mch_id'=>$this->mch_id,
            'nonce_str'=>$this->getNonceStr(16),
            'sign_type'=>'MD5',
            'body'=>$body,
            'out_trade_no'=>$no,
            'total_fee'=>$total_fee,
            'spbill_create_ip'=>'115.29.165.48',
            'notify_url'=>$notify_url,
            'trade_type'=>'JSAPI',
        ];
        if (!empty($openid)){
            $params['openid']=$openid;
        }
        if (!empty($sub_openid)){
            $params['sub_openid']=$sub_openid;
        }
        if (!empty($this->sub_mch_id)){
            $params['sub_mch_id']=$this->sub_mch_id;
        }
        if (!empty($this->sub_appid)){
            $params['sub_appid']=$this->sub_appid;
            unset($params['openid']);
            $params['sub_openid']=$openid;
        }

        $params['sign']=$this->getSign($params,$this->key);

        $this->getHttp()->setUrl($url);
        $xml=$this->getHttp()->post($this->toXml($params));
        $result = json_decode(json_encode(simplexml_load_string($xml, 'SimpleXMLElement', LIBXML_NOCDATA)), true);

        if (empty($result['prepay_id'])){
            //throw new \Exception();
            if (!empty($result['err_code_des'])){
                throw new \Exception($result['err_code_des']);
            }elseif (!empty($result['return_msg'])){
                throw new \Exception($result['return_msg']);
            }else{
                throw new \Exception('接口返回未知错误');
            }
        }

        $jsPayParams=[
            'appId'=>$this->appid,
            'timeStamp'=>time()."",
            'nonceStr'=>$this->getNonceStr(16),
            'package'=>'prepay_id='.$result['prepay_id'],
            'signType'=>'MD5',
        ];
        if (!empty($this->sub_appid)){
            $jsPayParams['appId']=$this->sub_appid;
        }
        $jsPayParams['paySign']=$this->getSign($jsPayParams,$this->key);
        return $jsPayParams;
    }

    /**
     * 查询交易结果
     * @author heyu 2020-01-19 11:08
     * @param string $no
     * @return array
     * @throws \Exception
     */
    public function query($no){
        $url='https://api.mch.weixin.qq.com/pay/orderquery';
        $params=[
            'appid'=>$this->appid,
            'mch_id'=>$this->mch_id,
            'nonce_str'=>$this->getNonceStr(16),
            'sign_type'=>'MD5',
            'out_trade_no'=>$no,
        ];
        if (!empty($this->sub_mch_id)){
            $params['sub_mch_id']=$this->sub_mch_id;
        }

        $params['sign']=$this->getSign($params,$this->key);

        $this->getHttp()->setUrl($url);
        $xml=$this->getHttp()->post($this->toXml($params));

        return json_decode(json_encode(simplexml_load_string($xml, 'SimpleXMLElement', LIBXML_NOCDATA)), true);
    }

    /**
     * 发起退款
     * @author heyu 2020-01-19 11:10
     * @param string $out_refund_no 退款订单号
     * @param string $out_trade_no 商户支付订单号
     * @param int $total_fee 原始支付订单总金额 单位分
     * @param int $refund_fee 退款金额 单位分
     * @return mixed
     * @throws \Exception
     */
    public function refund($out_refund_no,$out_trade_no,$total_fee,$refund_fee){
        $url='https://api.mch.weixin.qq.com/secapi/pay/refund';
        $params=[
            'appid'=>$this->appid,
            'mch_id'=>$this->mch_id,
            'nonce_str'=>$this->getNonceStr(16),
            'sign_type'=>'MD5',
            'out_trade_no'=>$out_trade_no,
            'out_refund_no'=>$out_refund_no,
            'total_fee'=>$total_fee,
            'refund_fee'=>$refund_fee,
        ];
        if (!empty($this->sub_mch_id)){
            $params['sub_mch_id']=$this->sub_mch_id;
        }
        if (empty($this->ssl_cert_path)||empty($this->ssl_key_path)){
            throw new \Exception('缺少证书文件');
        }

        $params['sign']=$this->getSign($params,$this->key);

        $http=$this->getHttp();
        $http->setUrl($url);
        $http->setIsSsl(true);
        $http->setSslCertPath($this->ssl_cert_path);
        $http->setSslKeyPath($this->ssl_key_path);

        $xml=$http->post($this->toXml($params));

        return json_decode(json_encode(simplexml_load_string($xml, 'SimpleXMLElement', LIBXML_NOCDATA)), true);
    }
}