<?php
/**
 * Created by PhpStorm.
 * User: roc
 * Date: 2019/11/14
 * Time: 20:10
 */

namespace kwy\wechatapi\wxcard\entity;


class WxCardUpdateEntity
{

    /**
     * 代金券
     */
    const CARD_TYPE_CASH='CASH';
    /**
     * 礼品券
     */
    const CARD_TYPE_GIFT='GIFT';
    /**
     * 团购券
     */
    const CARD_TYPE_GROUPON='GROUPON';
    /**
     * 会员卡
     */
    const CARD_TYPE_MEMBER_CARD='MEMBER_CARD';
    /**
     * 普通优惠券
     */
    const CARD_TYPE_GENERAL_COUPON='GENERAL_COUPON';


    /**
     * 有效期 类型
     */
    const DATE_TYPE_FIX_TIME_RANGE='DATE_TYPE_FIX_TIME_RANGE';
    /**
     * 有效期 领取后有效
     */
    const DATE_TYPE_FIX_TERM='DATE_TYPE_FIX_TERM';

    private $card_type='';

    /**
     * 卡券基础属性
     * @var array
     */
    protected $base_info=[
        /*'logo_url'=>'',
        'brand_name'=>'',
        'code_type'=>'CODE_TYPE_QRCODE',
        'title'=>'',
        'color'=>'Color010',
        'notice'=>'',
        'service_phone'=>'',
        'description'=>'',
        'date_info'=>[],
        'sku'=>[
            'quantity'=>10000,
        ],
        "use_limit"=>100,
        "get_limit"=> 3,
        "use_custom_code"=> false,
        "bind_openid"=> false,
        "can_share"=> true,
        "can_give_friend"=> true,
        "location_id_list"=> [
            123,
        ],
        "center_title"=> "顶部居中按钮",
        "center_sub_title"=> "按钮下方的wording",
        "center_url"=> "www.qq.com",
        "custom_url_name"=> "立即使用",
        "custom_url"=> "http=>//www.qq.com",
        "custom_url_sub_title"=> "6个汉字tips",
        "promotion_url_name"=> "更多优惠",
        "promotion_url"=> "http=>//www.qq.com",
        "source"=> "大众点评"*/
    ];

    /**
     * @var array 卡券高级属性
     */
    protected $advanced_info=[
        /*'use_condition'=>[

        ],
        'abstract'=>[
            'abstract'=>'',
            'icon_url_list'=>[],

        ],
        'text_image_list'=>[

        ],
        'time_limit'=>[

        ],
        'business_service'=>[

        ],*/
    ];

    /**
     * @var array 卡券特有属性
     */
    protected $card_info=[];



    /**
     * 卡券类型  优惠券 GENERAL_COUPON  代金券 CASH, 团购券 GROUPON 礼品券GIFT
     * @author heyu 2019-11-01 14:02
     * @param string $type
     * @return $this
     */
    public function setCardType($type){
        $this->card_type=$type;
        return $this;
    }

    /**
     * 设置商户logo图片 卡券的商户logo，建议像素为300*300
     * @author heyu 2019-11-01 17:20
     * @param string $logo_url
     * @return $this
     */
    public function setLogoUrl($logo_url){
        $this->base_info['logo_url']=$logo_url;
        return $this;
    }

    /**
     * 设置卡券里的 品牌名称 商户名称信息  商户名字,字数上限为12个汉字
     * @author heyu 2019-11-01 18:03
     * @param string $brand_name 最多12个字
     * @return $this
     */
    public function setBrandName($brand_name){
        $this->base_info['brand_name']=$brand_name;
        return $this;
    }

    /**
     * 设置优惠券名称  卡券名，字数上限为9个汉字。(建议涵盖卡券属性、服务及金额)。
     * @author heyu 2019-11-01 18:33
     * @param string $title
     * @return $this
     */
    public function setTitle($title){
        $this->base_info['title']=$title;
        return $this;
    }

    /**
     * 设置卡券库存
     * @author heyu 2019-11-04 16:28
     * @param int $num
     * @return $this
     */
    public function setSku($num){
        $this->base_info['sku']['quantity']=$num;
        return $this;
    }

    /**
     * 设置 优惠券有效期 自领取后多少天内有效
     * @author heyu 2019-11-01 18:22
     * @param int $fixed_term 表示自领取后多少天内有效，不支持填写0
     * @param int $fixed_begin_term 表示自领取后多少天开始生效，领取后当天生效填写0
     * @return $this
     */
    public function setDateTypeFixTerm($fixed_term,$fixed_begin_term){
        $dataInfo=[
            'type'=>self::DATE_TYPE_FIX_TERM,
            'fixed_term'=>$fixed_term,
            'fixed_begin_term'=>$fixed_begin_term,
        ];
        $this->base_info['date_info']=$dataInfo;
        return $this;
    }

    /**
     * 设置 优惠券有效期 自领取后多少天内有效
     * @author heyu 2019-11-01 18:22
     * @param int $begin_timestamp 表示起用时间 从1970年1月1日00:00:00至起用时间的秒数，最终需转换为字符串形态传入。（东八区时间,UTC+8，单位为秒）
     * @param int $end_timestamp 表示结束时间  建议设置为截止日期的23:59:59过期 。 （ 东八区时间,UTC+8，单位为秒 ）
     * @return $this
     */
    public function setDateTypeFixTimeRange($begin_timestamp,$end_timestamp){
        $dataInfo=[
            'type'=>self::DATE_TYPE_FIX_TERM,
            'begin_timestamp'=>$begin_timestamp,
            'end_timestamp'=>$end_timestamp,
        ];
        $this->base_info['date_info']=$dataInfo;
        return $this;
    }

    /**
     * 设置 启用门槛
     * @author heyu 2019-11-01 16:12
     * @param int $value
     * @return $this
     */
    public function setLeastCost($value){
        #TODO
        return $this;
    }

    /**
     * 设置团购券介绍
     * @author heyu 2019-11-01 13:55
     * @param string(3072) $info
     * @return $this
     */
    public function setDealDetail($info){
        $this->card_info['deal_detail']=$info;
        return $this;
    }

    /**
     * 设置子商户 第三方代制模式
     * @author heyu 2019-11-01 18:51
     * @param string $merchant_id
     * @return $this
     */
    public function setMerchantInfo($merchant_id){
        $this->base_info['sub_merchant_info']=['merchant_id'=>$merchant_id];
        return $this;
    }

    /**
     * 设置使用条件 允许使用类目信息
     * @author heyu 2019-11-01 13:45
     * @param string $info
     */
    public function setAcceptUseCondition($info){
        $this->advanced_info['use_condition']['accept_category']=$info;
        return $this;
    }

    /**
     * 设置使用条件 拒绝使用的类目信息
     * @author heyu 2019-11-01 13:45
     * @param string $info
     * @return $this
     */
    public function setRejectUseCondition($info){
        $this->advanced_info['use_condition']['reject_category']=$info;
        return $this;
    }

    /**
     * 设置 是否允许和其他折扣一起使用
     * @author heyu 2019-11-01 13:50
     * @param bool $can
     * @return $this
     */
    public function setCanUseWithOtherDiscount(bool $can){
        $this->advanced_info['use_condition']['reject_category']=$can;
        return $this;
    }

    /**
     * 设置使用须知
     * @author roc
     * @param string $description 使用须知
     * @throws \Exception
     * @return $this
     */
    public function setDescription(string $description=''){
        $len=mb_strlen($description,'utf-8');
        if ($len > 1024) throw new \Exception('使用须知文字内容过多');
        $this->base_info['description'] = $description;
        return $this;
    }

    /**
     * 设置商家服务电话
     * @author roc
     * @param string $service_phone 商家电话
     * @return $this
     */
    public function setServicePhone(string $service_phone=''){
        $this->base_info['service_phone'] = $service_phone;
        return $this;
    }

    /**
     * 设置卡券顶部数据
     * @author roc
     * @param string $title 卡券顶部居中的按钮文字 ，仅在卡券状态正常(可以核销)时显示。
     * @param string $sub_title 卡券显示在入口下方的提示语 ，仅在卡券状态正常(可以核销)时显示。
     * @param string $url 卡券顶部居中的url ，仅在卡券状态正常(可以核销)时显示。
     * @return $this
     */
    public function setCenter(string $title='',string $sub_title='',string $url=''){
        if(!empty($title)) $this->base_info['center_title'] = $title;
        if(!empty($sub_title)) $this->base_info['center_sub_title'] = $sub_title;
        if(!empty($url)) $this->base_info['center_url'] = $url;
        return $this;
    }

    /**
     * 设置自定义跳转外链
     * @author roc
     * @param string $custom_url_name 自定义跳转外链的入口名字。
     * @param string $custom_url_sub_title 显示在入口右侧的提示语。
     * @param string $custom_url 自定义跳转的URL。
     * @return $this
     */
    public function setCustom(string $custom_url_name='',string $custom_url_sub_title='',string $custom_url=''){
        if(!empty($custom_url_name)) $this->base_info['custom_url_name'] = $custom_url_name;
        if(!empty($custom_url_sub_title)) $this->base_info['custom_url_sub_title'] = $custom_url_sub_title;
        if(!empty($custom_url)) $this->base_info['custom_url'] = $custom_url;
        return $this;
    }

    /**
     * 设置下方自定义推广入口
     * @author roc
     * @param string $promotion_url_name 营销场景的自定义入口名称。
     * @param string $promotion_url_sub_title 显示在营销入口右侧的提示语。
     * @param string $promotion_url 入口跳转外链的地址链接。
     * @return $this
     */
    public function setPromotionLink(string $promotion_url_name,string $promotion_url_sub_title,string $promotion_url){
        if(!empty($promotion_url_name)) $this->base_info['promotion_url_name'] = $promotion_url_name;
        if(!empty($promotion_url_sub_title)) $this->base_info['promotion_url_sub_title'] = $promotion_url_sub_title;
        if(!empty($promotion_url)) $this->base_info['promotion_url'] = $promotion_url;
        return $this;
    }

    /**
     * 设置图文消息介绍
     * @author roc
     * @param array $text_image_list    [
     *                                      [
     *                                          'image_url' =>  '上传图片获得链接，并在此填入，否则报错',
     *                                          'text'      =>  '图文描述',
     *                                      ],
     *                                      [
     *                                          'image_url' =>  '上传图片获得链接，并在此填入，否则报错',
     *                                          'text'      =>  '图文描述',
     *                                      ]
     *                                  ]
     * @return $this
     */
    public function setTextImageList($text_image_list){
        $this->advanced_info['text_image_list'] = $text_image_list;
        return $this;
    }

    /**
     * 设置服务项
     * @author roc
     * @param array $services
     * @return $this
     */
    public function setService(array $services){
        $service_list=[];
        foreach ($services as $v){
            $service_list[]='BIZ_SERVICE_'.$v;
        }
        $this->advanced_info['business_service']=$service_list;
        return $this;
    }

    /**
     * 获取请求数据
     * @author roc
     * @return mixed
     */
    public function getData(){
        $type_lower=strtolower($this->card_type);
        $data[$type_lower]=[
            'base_info'=>$this->base_info,
            'advanced_info'=>$this->advanced_info,
        ];
        $data[$type_lower]=array_merge($data[$type_lower],$this->card_info);
        return $data;
    }
}