<?php
/**
 *
 * Auther: heyu
 * DateTime: 2019-11-04 17:23
 *
 */

namespace kwy\wechatapi\wxcard\entity;


class WxMemberCardEntity extends WxCardEntity
{

    /**
     * 开卡字段信息
     * @var array
     */
    protected $activate_info=[
        'required_form'=>[
            'can_modify'=>false,
            'rich_field_list'=>[

            ],
            'common_field_id_list'=>[

            ],
        ],
        'optional_form'=>[
            'can_modify'=>false,
            'common_field_id_list'=>[

            ],
        ],

    ];

    public function __construct()
    {
        $this->setCardType(WxCardEntity::CARD_TYPE_MEMBER_CARD);
        $this->card_info=[
            'supply_bonus'=>true,
            'supply_balance'=>false,
            'prerogative'=>'',
            'auto_activate'=>true,
            'activate_url'=>'',
        ];
    }

    /**
     * 设置是否支持积分功能
     * @author heyu 2019-11-04 16:41
     * @param bool $supply
     * @return $this
     */
    public function setSupplyBonus(bool $supply){
        $this->card_info['supply_bonus']=$supply;
        return $this;
    }

    /**
     * 设置背景图
     * @author roc
     * @param string $pic_url 背景图片地址(微信cdn地址)
     * @return $this
     */
    public function setBackGroundUrl(string $pic_url){
        $this->card_info['background_pic_url'] = $pic_url;
        return $this;
    }

    /**
     * 设置会员卡自动激活
     * @author heyu 2019-11-04 17:35
     * @param bool $auto 是否自动激活
     * @return $this
     */
    public function setAutoActivate(bool $auto){
        $this->card_info['auto_activate']=$auto;
        return $this;
    }

    /**
     * 设置微信一键激活 默认关闭
     * @author heyu 2019-11-04 17:45
     * @param bool $wx_activate
     * @return $this
     */
    public function setWxActivate(bool $wx_activate){
        if ($wx_activate){
            unset($this->card_info['activate_url']);
        }
        $this->card_info['wx_activate']=$wx_activate;
        return $this;
    }

    /**
     * 会员特权设置
     * @author heyu 2019-11-04 16:57
     * @param string $info
     * @return $this
     */
    public function setPrerogative($info){
        $this->card_info['prerogative']=$info;
        return $this;
    }

    /**
     * 永久有效期 类型
     * @author heyu 2019-11-04 17:11
     * @return $this
     */
    public function setDateTypePermanent(){
        $this->base_info['date_info']=['type'=>'DATE_TYPE_PERMANENT'];
        return $this;
    }

    /**
     * 设置激活时 展示的会员守则 会员信息使用申明等信息
     * @author heyu 2019-11-04 18:36
     * @param string $title
     * @param string $url
     * @return $this
     */
    public function setServiceStatement($title,$url){
        $this->activate_info['service_statement']=[
            'name'=>$title,
            'url'=>$url,
        ];
        return $this;
    }

    /**
     * 添加会员信息 公共字段信息
     * @author heyu 2019-11-04 18:54
     * @param string $field_id 见MemberCardField类 成员
     * @param bool $is_required 是否必填字段 否则是可选字段
     * @return $this
     */
    public function addCommonFieldId($field_id,$is_required){
        $field_type=$is_required?'required_form':'optional_form';

        if (!in_array($field_id,$this->activate_info[$field_type]['common_field_id_list'])){
            $this->activate_info[$field_type]['common_field_id_list'][]=$field_id;
        }
        return $this;
    }


    /**
     * 设置 激活时 展示老会员绑定入口
     * @author heyu 2019-11-04 18:38
     * @param $title
     * @param $url
     * @return $this
     */
    public function setBindOldCard($title,$url){
        $this->activate_info['bind_old_card']=[
            'name'=>$title,
            'url'=>$url,
        ];
        return $this;
    }

    /**
     * 设置跳转式 一键开卡 该模式用户在原生界面填写会员信息后 跳转到商户自定义url上 并携带用户信息
     * @author heyu 2019-11-05 09:44
     * @param string $url 举例 如填写 url为www.qq.com,则拼接后的url为 www.qq.com&card_id=pbLatjvFdsLDUMoN8JqcsGeiMHKk&encrypt_code=Bupk8bb9xxxxxx3rdXV6fClBVtkHQplYohdzGvgDl4%3D&outer_str=&openid=obLatjjwDxxxxxxxoGIdwNqRXw&activate_ticket=fDZv9eMQAFfrNr3XBoqhb%2F%2BMSDM0yjDF6CdiUhC%2BOlEaxb0clsUxxxxxxxxxxxd6yQsjRMRu4kAcKTibYLN5tmHBdll1b6zQRsLF53MpKjGU%3D
     * @return $this
     */
    public function setWxActivateAfterSubmit($url){
        $this->card_info['wx_activate']=true;
        $this->card_info['wx_activate_after_submit']=true;
        $this->card_info['wx_activate_after_submit_url']=$url;
        return $this;
    }

    /**
     * 设置会员卡下方自定义推广链接（自定义会员信息类目，会员卡激活后显示。）
     * @author roc
     * @param string $name 使用入口
     * @param string $tips 提示文字：激活后显示
     * @param string $url 跳转地址
     * @return $this
     */
    public function setCustomCell1(string $name='',string $tips='',string $url=''){
        if(!empty($name)) $this->card_info['custom_cell1']['name'] = $name;
        if(!empty($tips)) $this->card_info['custom_cell1']['tips'] = $tips;
        if(!empty($url)) $this->card_info['custom_cell1']['url'] = $url;
        return $this;
    }

    /**
     * 设置会员卡自定义会员信息类目1，会员卡激活后显示,包含name_type (name) 和url字段
     * @author roc
     * @param string $name 会员信息类目自定义名称，当开发者变更这类类目信息的value值时 不会触发系统模板消息通知用户
     * @param string $url 点击类目跳转外链url
     * @param string $name_type 会员信息类目半自定义名称，当开发者变更这类类目信息的value值时 可以选择触发系统模板消息通知用户。
     *                      FIELD_NAME_TYPE_LEVEL              等级 FIELD_NAME_TYPE_COUPON        优惠券                 FIELD_NAME_TYPE_STAMP            印花 FIELD_NAME_TYPE_DISCOUNT      折扣 FIELD_NAME_TYPE_ACHIEVEMEN  成就 FIELD_NAME_TYPE_MILEAGE          里程 FIELD_NAME_TYPE_SET_POINTS     集点 FIELD_NAME_TYPE_TIMS                次数
     * @return $this
     */
    public function setCustomField1(string $name='',string $url='',string $name_type=''){
        if(!empty($name)) $this->card_info['custom_field1']['name'] = $name;
        if(!empty($url)) $this->card_info['custom_field1']['url'] = $url;
        if(!empty($name_type)) $this->card_info['custom_field1']['name_type'] = $name_type;
        return $this;
    }

    /**
     * 设置会员卡自定义会员信息类目2，会员卡激活后显示,包含name_type (name) 和url字段
     * @author roc
     * @param string $name 会员信息类目自定义名称，当开发者变更这类类目信息的value值时 不会触发系统模板消息通知用户
     * @param string $url 点击类目跳转外链url
     * @param string $name_type 会员信息类目半自定义名称，当开发者变更这类类目信息的value值时 可以选择触发系统模板消息通知用户。
     *                      FIELD_NAME_TYPE_LEVEL              等级 FIELD_NAME_TYPE_COUPON        优惠券
     * @return $this
     */
    public function setCustomField2(string $name='',string $url='',string $name_type=''){
        if(!empty($name)) $this->card_info['custom_field2']['name'] = $name;
        if(!empty($url)) $this->card_info['custom_field2']['url'] = $url;
        if(!empty($name_type)) $this->card_info['custom_field2']['name_type'] = $name_type;
        return $this;
    }

    /**
     * 设置会员卡自定义会员信息类目3，会员卡激活后显示,包含name_type (name) 和url字段
     * @author roc
     * @param string $name 会员信息类目自定义名称，当开发者变更这类类目信息的value值时 不会触发系统模板消息通知用户
     * @param string $url 点击类目跳转外链url
     * @param string $name_type 会员信息类目半自定义名称，当开发者变更这类类目信息的value值时 可以选择触发系统模板消息通知用户。
     *                      FIELD_NAME_TYPE_LEVEL              等级 FIELD_NAME_TYPE_COUPON        优惠券
     * @return $this
     */
    public function setCustomField3(string $name='',string $url='',string $name_type=''){
        if(!empty($name)) $this->card_info['custom_field3']['name'] = $name;
        if(!empty($url)) $this->card_info['custom_field3']['url'] = $url;
        if(!empty($name_type)) $this->card_info['custom_field3']['name_type'] = $name_type;
        return $this;
    }

    //获取开卡信息
    public function getActivateData(){
        return $this->activate_info;
    }
}