<?php
/**
 * Created by PhpStorm.
 * User: roc
 * Date: 2019/11/12
 * Time: 15:02
 */

namespace kwy\wechatapi\wxresource;

use kwy\util\http\CurlHttp;

/**
 * 微信资源文件上传类
 * Class WxResource
 * @package kwy\wechatapi\wxresource
 */
class WxResource
{

    /**
     * @var CurlHttp
     */
    private $httpClient;

    private $accessToken;

    public function setAccessToken($accessToken){
        $this->accessToken=$accessToken;
    }

    /**
     * 上传商户图片至微信服务器，获取相应logo_url/icon_list/image_url，用于卡券创建  图片是全局外网url地址
     * 1.上传的图片限制文件大小限制1MB，仅支持JPG、PNG格式。
     * 2.调用接口获取图片url仅支持在微信相关业务下使用。
     * @author roc
     * @param string $path 文件相对服务器的路径
     * @return mixed 微信url地址
     * @throws \Exception
     */
    public function uploadImg($path){
        $url = 'https://api.weixin.qq.com/cgi-bin/media/uploadimg?access_token=ACCESS_TOKEN';

        if (!file_exists($path)){
            throw new \Exception($path.'文件不存在');
        }
        $c_file = new \CURLFile($path);
        $data = ['imgfile'=>$c_file];
        $ret = $this->getHttpClient()
//            ->setProxy('127.0.0.1:8888')
            ->setUrl($this->getUrlWithAccessToken($url))
            ->setContentType("multipart/form-data")
            ->post($data);

        $ret_arr=json_decode($ret,true);
        if (empty($ret_arr['url'])){
            throw new \Exception($ret_arr['errmsg']);
        }
        return $ret_arr['url'];
    }

    /**
     * 获取携带access_token的地址
     * @author roc
     * @param string $url 请求地址
     * @return mixed
     * @throws \Exception
     */
    private function getUrlWithAccessToken($url){
        if (empty($this->accessToken)){
            throw new \Exception('accessToken 未设置');
        }
        return str_replace('ACCESS_TOKEN',$this->accessToken,$url);
    }

    public function getHttpClient(){
        if (empty($this->httpClient)){
            $this->httpClient = new CurlHttp();
            //$this->httpClient->setProxy('127.0.0.1:8888');
            $this->httpClient->setContentType('application/json');
        }
        return $this->httpClient;
    }
}